;------------------------------------------------------------------------------
; Projeto - I2C-soft_drZ80
; Arquivo - I2C-soft_drZ80.asm
; Desenv. - Eng. Márcio José Soares
; Versão  - 1.0
; Data    - 15/03/2023
;
; Permite enviar/receber um byte via I2C por software, sem uso de CI's especificos
; usando a PPI8255
;
; Compilador: pasmo -d -v I2C-soft_drZ80.asm I2C-soft_drZ80.bin > I2C-soft_drZ80.lst
;
; Pinos:
; PPI8255
;
; Pinos:
; PPI8255
; ------------------
; DT OUT    PA0
; DT IN	 	PC0
; CK        PA1
;
; CS_WR	 	PB0 - seleciona DT OUT (1) ou DT IN (0)
;		
;------------------------------------------------------------------------------
;
; Últimas alterações:
;
; em 15/04/2023:
;   - criado esse arquivo/projeto
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; Declara endereços dos registradores do 8255 na placa
;
_8255_OA        equ     0010H           ;endereco para entrada/saida no portA do 8255
_8255_OB        equ     0011H           ;endereco para entrada/saida no portB do 8255
_8255_OC        equ     0012H           ;endereco para entrada/saida no portC do 8255

;------------------------------------------------------------------------------
; Declara funções necessárias na placa drZ80_uP
;
wrB8255A        equ     0085H           ; escreve A no portB 
wrA8255A        equ     0088H           ; escreve A no portA
rd8255C         equ     008BH           ; lê do portC e insere no end de memória 87CEH
t_50us          equ     0094H           ; temporiza 50us
t_5ms           equ     009AH			; temporiza 5ms
t_50ms          equ     00A3H           ; temporiza 50ms
t_100ms         equ     00A6H			; temporiza 100ms
t_500ms         equ     00A9H           ; temporiza 500ms
wrzerodisp      equ     007CH           ; limpa display 7 segmentos - coloca zeros
wrAdisp5_6		equ 	007FH			; escreve nos disps 5 e 6 conteúdo de A
wrAdisp1_6		equ     0082H			; escreve nos displays de 1 a 6 o valor de A

;-------------------------------------------------------------------------------
; Declara constantes
;
CS_WR			equ   	01H				; habilita buffer DT out
CS_RD           equ     0FEH			; habilita buffer DT in

HDT				equ     01H				; pino de dados e clock - lógica
LDT             equ     0FEH
HCK				equ		02H
LCK				equ		0FDH

;-------------------------------------------------------------------------------
; Declara endereços I2C
;
I2CADDWR		equ     27H             ; Escrita - A0,A1 e A2 = 1
I2CADDRD		equ		26H				; Leitura - A0 = 0, A1 e A2 = 1
										
;-------------------------------------------------------------------------------
; Declara variáveis
;
                org     8000H           ; endereço inicial da RAM e programa do usuário
                jr      main            ; desvia para main
                
myout           db      1               ; var de saida para dados
mydata          db      1               ; var de entrada de dados
myflags			db		1				; bit 0 - 1 para envio, 0 para recepção
I2Cadd			db		1				; var para endereço I2C

;-------------------------------------------------------------------------------
; Subrotina principal
;
main:		call	t_50ms			; aguarda iniciar
			ld      a,CS_WR         ; habilita DT out
			call    wrB8255A		; envia para port
			ld      a,03H           ; levanta clock e dados
			call    wrA8255A		; envia para port
			call    t_5ms           ; aguarda
			
            call	wrzerodisp		; mantem display da placa zerado      
                            
            ld      hl,I2Cadd       ; pega end da var
            ld      a,I2CADDWR		; pega valor do endereço I2C 
            rl		a				; shit a esquerda de 1 - 27H = 4EH
            ld      (hl),a          ; carrega valor na var
            
			ld		hl,myflags		; pega end da var
			set 	0,(hl)          ; avisa que é envio
                
main01:		ld		hl,myout		; pega end da var
			ld      a,01H			; coloca valor em A
			ld      (hl),a          ; valor inicial agora na var 
			
			ld      b,08H           ; faz 8 vezez 
			
loop:		call    sndrecI2C       ; envia dado em myout via I2C
			
			ld		hl,myout		; pega end da var
			ld      a,(hl)			; pega dado
            
            push	bc
            push 	hl
            ld		c,a				; salva
            and		0F0H			; limpa LSB
            rr      a				; coloca MSB no LSB
            rr      a
            rr      a
            rr      a
            
            ld		b,3
            call	wrAdisp1_6		; escreve A no display 3
            
            ld      a,c				; recupera a anterior
            and		0FH				; limpa MSB
            
            ld		b,4
            call	wrAdisp1_6		; escreve A no display 4
            pop     hl
           	pop		bc
						
			call    t_250ms      	; aguarda
						
			ld      hl,I2Cadd       ; pega end da var
            ld      a,I2CADDRD		; pega valor do endereço I2C 
            rl		a				; shit a esquerda de 1 - 26H = 4CH
            or      01H			 	; bit 0 setado para leitura
            ld      (hl),a          ; carrega valor na var
            
			ld		hl,myflags		; pega end da var
			res 	0,(hl)          ; avisa que é recepção
			call    sndrecI2C       ; recebe dado em mydata via I2C			
			
			ld		hl,mydata
			ld		a,(hl)
			call	wrAdisp5_6		; escreve a var em disp 5 e 6
			
            call    t_250ms      	; aguarda
            
            ld      hl,I2Cadd       ; pega end da var
            ld      a,I2CADDWR		; pega valor do endereço I2C 
            rl		a				; shit a esquerda de 1 - 27H = 4EH
            ld 		(hl),a
            
            ld		hl,myflags		; pega end da var
			set 	0,(hl)          ; avisa que é envio
			
			ld		hl,myout		; pega end da var
            rl		(hl)			; gira a esquerda
            
            djnz    loop            ; decrementa b e continua se não zero 
            jr      main01          ; recarrega novamente e faz eternamente
                   
;-------------------------------------------------------------------------------
; Subrotina para enviar dado via I2C usando a PPI8255
; byte a enviar em myout
; endereço I2C em I2Cadd
;
sndrecI2C:	push	hl
			push	bc
			
			ld		a,CS_WR		    ; habilita buffer DT out
			out		(_8255_OB),a 	; envia
			
			ld      hl,myflags		; pega end da var
			bit     0,(hl)			; testa bit para saber se recebe ou envia
			jr      nz,sndI2C       ; bit setado, é envio
			
recI2C:		ld      a,01H			; envia apenas endereço
			ld      c,a             ; carrega c 
			jr      sndstart
		
sndI2C:		ld      a,02H			; guia para endereço e dado
			ld      c,a				; envia endereço e dado
			
sndstart:	ld   	a,02H			; abaixa pino de dado
			out     (_8255_OA),a    ; coloca bits no portA
			nop
			ld      a,00H           ; abaixa agora o clock também
			out     (_8255_OA),a    ; coloca bits no portA
			nop
			nop
			
sndaddress: ld		hl,I2Cadd		; endereço I2C
			ld      a,(hl)
			ld		h,a				; endeço agora em h
			jr      sndbyte			; desvia para enviar 8 bits
		
snddata:    ld		hl,myout		; dado I2C
			ld      a,(hl)
			ld		h,a				; dado agora em h
						
sndbyte:	ld		b,08H			; são 8 bits a enviar
scont:		bit     7,h				; testa o bit 7
			jr      nz,eh1			; bit igual a 1
	
eh0:		ld		a,00H			; carrega 0 ou 1 	
			jr      sndclk			; continua
eh1:		ld		a,01H

sndclk: 	out		(_8255_OA),a 	; coloca bit DT
			nop
			or		HCK	
			out		(_8255_OA),a 	; sobe clock
			nop
			nop
			nop
			and     LCK     		; 
			out		(_8255_OA),a 	; desce clock
			rl		h				; gira byte
			djnz    scont			; continua enviar byte

getack:		or      HCK				; levanta clock - pega ack slave
			out		(_8255_OA),a 	; envia para port
			nop
			nop
			nop
			and     LCK				; abaixa clock
			out		(_8255_OA),a 	; envia para port
            
			dec     c
			jr      nz,snddata      ; se não é zero, envia dado    
						
			ld		hl,myflags		; pega end da var
			bit     0,(hl)          ; testa bit para saber se era envio
			jr      nz,sndstop      ; envia stop caso seja envio
						
			
recbyte:	ld		a,CS_RD		    ; habilita buffer DT in
			out		(_8255_OB),a 	; prepara para receber bits
						
		    ld      b,8				; serão 8 bits
			ld      a,00H			; zera 
			ld      h,a      		; byte ficará em h
			jr		recCLK			; desvia para receber

giraH:		rl      h				; gira para direita, mas o último não!!!
recCLK:		ld		a,HCK			; levanta clock
			out		(_8255_OA),a 	; envia para port
			nop
			in      a,(_8255_OC)	; pega bit
			ld      c,a
			ld      a,00H			; abaixa clock
			out		(_8255_OA),a 	; envia para port
			ld      a,c             ; restaura
			bit     0,a             ; testa bit
			jr      nz,eh1rec       ; se bit 1 desvia

eh0rec:     res     0,h             ; zera bit
			jr      contrec

eh1rec:     set     0,h				; set bit
contrec:    djnz	giraH			; desvia e gira até receber todos bits
									; mas quando b=1, sai sem girar!!!
									
			ld      a,h				; salva h em d
			ld      d,a
									
			ld		a,CS_WR		    ; habilita buffer DT out
			out		(_8255_OB),a 	; envia para port
			
			ld      a,01H			; levanda dado para nack
			out		(_8255_OA),a 	; envia para port
			nop
			or      HCK				; levanta clock
			out		(_8255_OA),a 	; envia para port
			nop
			nop
			and     LCK				; abaixa clock
			out		(_8255_OA),a 	; envia para port
			nop
			
sndstop:	ld		a,02H			; abaixa dado mantendo clock alto
			out		(_8255_OA),a 	; envia para port
			nop
			or		01H				; levanta dado também
			out		(_8255_OA),a 	; envia para port
			
			ld		hl,myflags		; pega end da var
			bit     0,(hl)          ; testa bit para saber se era envio
			jr      nz,sndend       ; envia stop caso seja envio
			
			ld      a,d				; guarda byte lido em a
			ld      hl,mydata       ; peda end da var
			ld      (hl),a          ; coloca byte na var 
			
sndend:		pop		bc
			pop     hl
			
			ret 					; retorna

;-------------------------------------------------------------------------------
; Subrotina de espera ocupada para aproximadamente 250ms
;
t_250ms:	call	t_100ms			; temporiza 100ms
			call	t_100ms			; temporiza +100ms
			call	t_50ms			; temporiza +50ms
			
			ret
	
	
;-------------------------------------------------------------------------------
; Fim do programa
;
			end						; fim!
